package function

import (
	"gitlab.smartblocklab.com/smartcontracts/sc-security/tools"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/actions"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/requests"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"encoding/json"
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
)

type GetAccountInfoByAccountID_Type func(externalAccountId string, adapter chaincode.StubInterfaceAdapter) (*model.AccountInfo, error)

func GetAccountInfoByAccountID_Function(accountId string, adapter chaincode.StubInterfaceAdapter) (*model.AccountInfo, error) {
	logger.Infof("find by accountId %s", accountId)
	// Invoke account check chaincode

	resp := tools.SignedInvokeChaincode(adapter, invoke.AccountChannel, actions.Actions().AccountCC().GetAccountInfoByAccountID, requests.Id{Id: accountId})
	// We got error status
	if resp.Status != int32(shim.OK) {
		logger.Errorf("verification of external account id %s failed, with following response %s", accountId, resp.Message)
		return nil, errors.New(resp.Message)
	}
	// No error, account verified
	var accountInfo model.AccountInfo
	err := json.Unmarshal(resp.Payload, &accountInfo)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to deserialize user account info, due to %s", err)
		logger.Error(errMsg)
		return nil, err
	}
	return &accountInfo, nil
}