package function

import (
	"gitlab.smartblocklab.com/smartcontracts/sc-security/tools"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/actions"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"fmt"
	"encoding/json"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"errors"
	"gitlab.smartblocklab.com/smartcontracts/common/model"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
)

// получение заблокированных средств по operationID
func FindLockAmountByOperationIDAndUserId(operationID string, userID string, adapter chaincode.StubInterfaceAdapter) (*model.FindLockAmountResult, error) {
	resp := tools.SignedInvokeChaincode(adapter, invoke.FindLockAmountByOperationIDAndUserIdChannelName,
		actions.Actions().TransactionCC().FindLockAmountByOperationIDAndUserId, model.FindLockAmountByOperationIDAndUserIdArg{
			OperationID: operationID,
			UserID:      userID,
		})

	if resp.Status != int32(shim.OK) {
		errMsg := fmt.Sprintf("Problem to find lock transaction by operationID %s and userID %s. Message: %s", operationID, userID, resp.Message)
		logger.Error(errMsg)
		return nil, errors.New(errMsg)
	}

	var findLockAmountResult model.FindLockAmountResult
	err := json.Unmarshal(resp.Payload, &findLockAmountResult)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to deserialize response from 'findLockAmountByOperationIDAndUserId' function, due to %s", err)
		logger.Error(errMsg)
		return nil, errors.New(errMsg)
	}

	return &findLockAmountResult, nil
}
