package function

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/tools"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/actions"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/requests"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"encoding/json"
	"fmt"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"errors"
)

type GetUser_Type func(userID string, adapter chaincode.StubInterfaceAdapter) (*model.User, error)

func GetUser_Function(userID string, adapter chaincode.StubInterfaceAdapter) (*model.User, error) {
	logger.Infof("userLoader (userID: %s)", userID)

	// Invoke account check chaincode
	resp := tools.SignedInvokeChaincode(adapter, invoke.GetUserChannel, actions.Actions().UserCC().GetUser, requests.Id{Id: userID})
	// We got error status
	if resp.Status != int32(shim.OK) {
		logger.Errorf("verification of user id %s failed, with following response %s", userID, resp.Message)
		return nil, errors.New(resp.Message)
	}
	// No error, account verified
	var user model.User
	err := json.Unmarshal(resp.Payload, &user)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to deserialize user, due to %s", err)
		logger.Error(errMsg)
		return nil, err
	}

	responseLogMsg := fmt.Sprintf("response: userId: %s, partnerCode: %s, customerType: %s, userStatus: %s", user.UserID, user.PartnerCode, user.CustomerType, user.UserStatus)
	logger.Error(responseLogMsg)
	return &user, nil
}
