package model

import (
	"encoding/json"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	pb "github.com/hyperledger/fabric/protos/peer"
)

// invoke_CheckExistsTransactionByList
type CheckExistsExternalTransactionStatus string

const (
	NOT_FOUND      CheckExistsExternalTransactionStatus = "NOT_FOUND"
	ALREADY_EXISTS CheckExistsExternalTransactionStatus = "ALREADY_EXISTS"
	ERROR          CheckExistsExternalTransactionStatus = "ERROR"
)

type CheckExistsExternalTransactionResult struct {
	ExternalTransactionID string                               `json:"external_transaction_id"`
	Status                CheckExistsExternalTransactionStatus `json:"status"`
	Message               string                               `json:"message"`
}

type CheckExistsResult struct {
	CheckExistsExternalTransactionResultList []CheckExistsExternalTransactionResult `json:"check_exists_external_transaction_result_list"`
}

type ExternalTransactionInfo struct {
	OperationID           string `json:"operation_id"`
	ExternalTransactionID string `json:"external_transaction_id"`
}

type SyncRequest struct {
	ExternalTransactionInfoList []ExternalTransactionInfo `json:"external_transaction_info_list"`
}

func (result CheckExistsResult) Marshal() pb.Response {
	bytes, e := json.Marshal(result)
	if e != nil {
		return shim.Error(e.Error())
	}

	return shim.Success(bytes)
}