package actions

import log "github.com/sirupsen/logrus"

type ActionIDsObject struct {
}

type ParameterType int

type Roles string

type SecurityPreference struct {
	ChaincodeName string
	FunctionName  string
	ParameterType ParameterType
	Roles         []Roles
}

var (
	logger = log.WithFields(log.Fields{})
)

func (securityPreference SecurityPreference) GetActionID() int32 {
	actionID := securityPreference.ChaincodeName + securityPreference.FunctionName
	logger.Infof("ActionId source: '%s'", actionID)
	var hash int32
	hash = Hash(actionID)
	logger.Infof("ActionId hash: '%d'", hash)
	return hash
}

func Hash(s string) int32 {
	var h int32
	h = 0
	var ln int32
	var li int32
	ln = int32(len(s))
	li = ln - 1 // li means last index
	var i int32
	for i = 0; i < ln; i++ {
		h += int32(s[i]) * Pow(31, li-i)
	}
	return h
}

func Pow(x int32, y int32) int32 {
	p := x
	if y == 0 {
		return 1
	}
	if y == 1 {
		return x
	}
	// if y < 0, panic?
	var i int32
	for i = 1; i < y; i++ {
		p *= x
	}
	return p
}

const UserParameter ParameterType = 1
const ServiceParameter ParameterType = 1 << 2
const ComplexParameter ParameterType = 1 << 3
const UnsecuredParameter ParameterType = 1 << 4
const BasicRole Roles = "USER"

var SecurityPreferenceArray = []SecurityPreference{
	SecurityPreference{"accountCC", "addWallet", ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "addExternalAccount", UserParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "addAccount", ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "activateWallet", UserParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "blockWallet", UserParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "activateAccount", UserParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "blockAccount", UserParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "verifyUserAccount", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "checkAndLoadAccount", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "getAllUserWallets", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "getAccountInfoByAccountID", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"accountCC", "getAccountByCurrencyType", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"authCC", "addAuthInfo", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"authCC", "updateAuthInfo", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"authCC", "getAuthInfo", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"commissionCC", "addCommission", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"commissionCC", "getCommission", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"commissionCC", "updateCommission", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"cryptocurrencyCC", "addCryptocurrencies", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"cryptocurrencyCC", "getCryptocurrenciesByDaemonName", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"cryptocurrencyCC", "clearTransferredCryptocurrencyList", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"externalOfferCC", "addOffer", ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{"externalOfferCC", "getUserOutgoingRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"externalOfferCC", "getRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"externalOfferCC", "getHistoryUserOutgoingRequests", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"externalOfferCC", "updateOfferExternalTransactionId", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"externalOfferCC", "getRequestsByStatusAndCurrencyType", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"externalOfferCC", "changeOfferStatusByOperationId", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"externalOfferCC", "changeOfferStatusByExternalTransactionId", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "addOffer", ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "getUserIncomingRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "getUserOutgoingRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "getHistoryUserIncomingRequests", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "getHistoryUserOutgoingRequests", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "getOffer", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "changeOfferStatusByOperationId", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "getRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "isOfferApproved", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "replayOffer", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"offerCC", "getReplayHistoryOffer", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "lockFundsOnAccount", ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "accountCurrencyBalance", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "transferFunds", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "getUsersTransaction", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "loadCryptocurrency", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "lockOutFundsOnAccount", ComplexParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "lockTransferToMarket", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "unlockFundsOnAccount", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "checkExistsTransactionByList", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "findLockAmountByOperationIDAndUserId", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"transactionCC", "loadMarketFounds", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "addRootUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "addOrganizationRootUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "addSystemUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "addServiceUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "addUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "getUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "addPublicKey", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "revokeUserPublicKey", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "verifyUserPublicKey", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"userCC", "verifyUserSignature", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"organizationCC", "addOrganization", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"organizationCC", "getOrganization", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"marketCC", "depositMarketFunds", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"marketCC", "getAllMarketTransactionsByUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"marketCC", "getMarketBalance", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"marketCC", "withdrawalMarketFunds", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"marketCC", "createMarketOrder", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"marketCC", "getAllMarketOrders", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"marketCC", "cancelMarketOrder", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"marketCC", "matchOrders", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"userPreferenceCC", "addPreference", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"userPreferenceCC", "getIntAccountByExtAccount", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},

	SecurityPreference{"systemPreferenceCC", "setFieldConstraints", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"systemPreferenceCC", "getFieldConstraints", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"systemPreferenceCC", "setEnabledCryptoCurrencies", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"systemPreferenceCC", "getEnabledCryptoCurrencies", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	SecurityPreference{"systemPreferenceCC", "setCryptoCurrencyConstraints", ServiceParameter, []Roles{BasicRole}},
	SecurityPreference{"systemPreferenceCC", "getCryptoCurrencyConstraints", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
}

// covered
func (obj ActionIDsObject) AccountCC() AccountCCObject {
	return AccountCCObject{
		SecurityPreference{"accountCC", "addWallet", ComplexParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "addExternalAccount", UserParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "addAccount", ComplexParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "activateWallet", UserParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "blockWallet", UserParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "activateAccount", UserParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "blockAccount", UserParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "verifyUserAccount", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "checkAndLoadAccount", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "getAllUserWallets", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "getAccountInfoByAccountID", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"accountCC", "getAccountByCurrencyType", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	}
}

// covered
func (obj ActionIDsObject) AuthCC() AuthCCObject {
	return AuthCCObject{
		SecurityPreference{"authCC", "addAuthInfo", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"authCC", "updateAuthInfo", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"authCC", "getAuthInfo", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	}
}

// covered
func (obj ActionIDsObject) CommissionCC() CommissionCCObject {
	return CommissionCCObject{
		SecurityPreference{"commissionCC", "addCommission", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"commissionCC", "getCommission", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"commissionCC", "updateCommission", ServiceParameter, []Roles{BasicRole}},
	}
}

// covered
func (obj ActionIDsObject) CryptocurrencyCC() CryptocurrencyCCObject {
	return CryptocurrencyCCObject{
		SecurityPreference{"cryptocurrencyCC", "addCryptocurrencies", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"cryptocurrencyCC", "getCryptocurrenciesByDaemonName", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"cryptocurrencyCC", "clearTransferredCryptocurrencyList", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	}
}

// covered
func (obj ActionIDsObject) ExternalOfferCC() ExternalOfferCCObject {
	return ExternalOfferCCObject{
		SecurityPreference{"externalOfferCC", "addOffer", ComplexParameter, []Roles{BasicRole}},
		SecurityPreference{"externalOfferCC", "getUserOutgoingRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"externalOfferCC", "getRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"externalOfferCC", "getHistoryUserOutgoingRequests", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"externalOfferCC", "updateOfferExternalTransactionId", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"externalOfferCC", "getRequestsByStatusAndCurrencyType", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"externalOfferCC", "changeOfferStatusByOperationId", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"externalOfferCC", "changeOfferStatusByExternalTransactionId", ServiceParameter, []Roles{BasicRole}},
	}
}

// covered
func (obj ActionIDsObject) OfferCC() OfferCCObject {
	return OfferCCObject{
		SecurityPreference{"offerCC", "addOffer", ComplexParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "getUserIncomingRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "getUserOutgoingRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "getHistoryUserIncomingRequests", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "getHistoryUserOutgoingRequests", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "getOffer", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "changeOfferStatusByOperationId", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "getRequestsByStatus", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "isOfferApproved", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "replayOffer", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"offerCC", "getReplayHistoryOffer", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	}
}

// covered
func (obj ActionIDsObject) TransactionCC() TransactionCCObject {
	return TransactionCCObject{
		SecurityPreference{"transactionCC", "lockFundsOnAccount", ComplexParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "accountCurrencyBalance", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "transferFunds", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "getUsersTransaction", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "loadCryptocurrency", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "lockOutFundsOnAccount", ComplexParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "lockTransferToMarket", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "unlockFundsOnAccount", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "checkExistsTransactionByList", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "findLockAmountByOperationIDAndUserId", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"transactionCC", "loadMarketFounds", ServiceParameter, []Roles{BasicRole}},
	}
}

// covered
func (obj ActionIDsObject) UserCC() UserCCObject {
	return UserCCObject{
		SecurityPreference{"userCC", "addRootUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "addOrganizationRootUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "addSystemUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "addServiceUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "addUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "getUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "addPublicKey", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "revokeUserPublicKey", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "verifyUserPublicKey", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"userCC", "verifyUserSignature", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	}
}

// covered
func (obj ActionIDsObject) OrganizationCC() OrganizationCCObject {
	return OrganizationCCObject{
		SecurityPreference{"organizationCC", "addOrganization", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"organizationCC", "getOrganization", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	}
}

func (obj ActionIDsObject) MarketCC() MarketCCObject {
	return MarketCCObject{
		SecurityPreference{"marketCC", "depositMarketFunds", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"marketCC", "getAllMarketTransactionsByUser", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"marketCC", "getMarketBalance", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"marketCC", "withdrawalMarketFunds", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"marketCC", "createMarketOrder", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"marketCC", "getAllMarketOrders", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SecurityPreference{"marketCC", "cancelMarketOrder", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"marketCC", "matchOrders", ServiceParameter, []Roles{BasicRole}},
	}
}

func (obj ActionIDsObject) UserPreferenceCC() UserPreferenceCCObject {
	return UserPreferenceCCObject{
		SecurityPreference{"userPreferenceCC", "addPreference", ServiceParameter, []Roles{BasicRole}},
		SecurityPreference{"userPreferenceCC", "getIntAccountByExtAccount", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	}
}

func (obj ActionIDsObject) SystemPreferenceCC() SystemPreferenceCCObject {
	return SystemPreferenceCCObject{
		SetFieldConstraints:          SecurityPreference{"systemPreferenceCC", "setFieldConstraints", ServiceParameter, []Roles{BasicRole}},
		GetFieldConstraints:          SecurityPreference{"systemPreferenceCC", "getFieldConstraints", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SetEnabledCryptoCurrencies:   SecurityPreference{"systemPreferenceCC", "setEnabledCryptoCurrencies", ServiceParameter, []Roles{BasicRole}},
		GetEnabledCryptoCurrencies:   SecurityPreference{"systemPreferenceCC", "getEnabledCryptoCurrencies", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
		SetCryptoCurrencyConstraints: SecurityPreference{"systemPreferenceCC", "setCryptoCurrencyConstraints", ServiceParameter, []Roles{BasicRole}},
		GetCryptoCurrencyConstraints: SecurityPreference{"systemPreferenceCC", "getCryptoCurrencyConstraints", ServiceParameter | UnsecuredParameter, []Roles{BasicRole}},
	}
}

type AccountCCObject struct {
	AddWallet                 SecurityPreference
	AddExternalAccount        SecurityPreference
	AddAccount                SecurityPreference
	ActivateWallet            SecurityPreference
	BlockWallet               SecurityPreference
	ActivateAccount           SecurityPreference
	BlockAccount              SecurityPreference
	VerifyUserAccount         SecurityPreference
	CheckAndLoadAccount       SecurityPreference
	GetAllUserWallets         SecurityPreference
	GetAccountInfoByAccountID SecurityPreference
	GetAccountByCurrencyType  SecurityPreference
}

type AuthCCObject struct {
	AddAuthInfo    SecurityPreference
	UpdateAuthInfo SecurityPreference
	GetAuthInfo    SecurityPreference
}
type CommissionCCObject struct {
	AddCommission    SecurityPreference
	GetCommission    SecurityPreference
	UpdateCommission SecurityPreference
}
type CryptocurrencyCCObject struct {
	AddCryptocurrencies                SecurityPreference
	GetCryptocurrenciesByDaemonName    SecurityPreference
	ClearTransferredCryptocurrencyList SecurityPreference
}
type ExternalOfferCCObject struct {
	AddOffer                                 SecurityPreference
	GetUserOutgoingRequestsByStatus          SecurityPreference
	GetRequestsByStatus                      SecurityPreference
	GetHistoryUserOutgoingRequests           SecurityPreference
	UpdateOfferExternalTransactionID         SecurityPreference
	GetRequestsByStatusAndCurrencyType       SecurityPreference
	ChangeOfferStatusByOperationID           SecurityPreference
	ChangeOfferStatusByExternalTransactionID SecurityPreference
}
type OfferCCObject struct {
	AddOffer                        SecurityPreference
	GetUserIncomingRequestsByStatus SecurityPreference
	GetUserOutgoingRequestsByStatus SecurityPreference
	GetHistoryUserIncomingRequests  SecurityPreference
	GetHistoryUserOutgoingRequests  SecurityPreference
	GetOffer                        SecurityPreference
	ChangeOfferStatusByOperationId  SecurityPreference
	GetRequestsByStatus             SecurityPreference
	IsOfferApproved                 SecurityPreference
	ReplayOffer                     SecurityPreference
	GetReplayHistoryOffer           SecurityPreference
}
type TransactionCCObject struct {
	LockFundsOnAccount                   SecurityPreference
	AccountCurrencyBalance               SecurityPreference
	TransferFunds                        SecurityPreference
	GetUsersTransaction                  SecurityPreference
	LoadCryptocurrency                   SecurityPreference
	LockOutFundsOnAccount                SecurityPreference
	LockTransferToMarket                 SecurityPreference
	UnlockFundsOnAccount                 SecurityPreference
	CheckExistsTransactionByList         SecurityPreference
	FindLockAmountByOperationIDAndUserId SecurityPreference
	LoadMarketFounds                     SecurityPreference
}
type UserCCObject struct {
	AddRootUser             SecurityPreference
	AddOrganizationRootUser SecurityPreference
	AddSystemUser           SecurityPreference
	AddServiceUser          SecurityPreference
	AddUser                 SecurityPreference
	GetUser                 SecurityPreference
	AddPublicKey            SecurityPreference
	RevokeUserPublicKey     SecurityPreference
	VerifyUserPublicKey     SecurityPreference
	VerifyUserSignature     SecurityPreference
}
type OrganizationCCObject struct {
	AddOrganization SecurityPreference
	GetOrganization SecurityPreference
}

type MarketCCObject struct {
	DepositMarketFunds             SecurityPreference
	GetAllMarketTransactionsByUser SecurityPreference
	GetMarketBalance               SecurityPreference
	WithdrawalMarketFunds          SecurityPreference
	CreateMarketOrder              SecurityPreference
	GetAllMarketOrders             SecurityPreference
	CancelMarketOrder              SecurityPreference
	MatchOrders                    SecurityPreference
}

type UserPreferenceCCObject struct {
	AddPreference             SecurityPreference
	GetIntAccountByExtAccount SecurityPreference
}

type SystemPreferenceCCObject struct {
	SetFieldConstraints          SecurityPreference
	GetFieldConstraints          SecurityPreference
	SetEnabledCryptoCurrencies   SecurityPreference
	GetEnabledCryptoCurrencies   SecurityPreference
	SetCryptoCurrencyConstraints SecurityPreference
	GetCryptoCurrencyConstraints SecurityPreference
}

func Actions() ActionIDsObject {
	return ActionIDsObject{}
}
