package tools

import (
	"encoding/json"
	"github.com/hyperledger/fabric/core/chaincode/shim"
	pb "github.com/hyperledger/fabric/protos/peer"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/actions"
	"gitlab.smartblocklab.com/smartcontracts/sc-security/requests"
)

type ChaincodeStubInterfaceAdapter interface {
	InvokeChaincode(chaincodeName string, args [][]byte, channel string) pb.Response
}

func InvokeChaincode_GetUser(stub ChaincodeStubInterfaceAdapter, UserID string) pb.Response {
	return SignedInvokeChaincode(stub, "userchannel", actions.Actions().UserCC().GetUser, requests.Id{Id: UserID})
}

func InvokeChaincode_VerifyUserSignature(stub ChaincodeStubInterfaceAdapter, arg interface{}) pb.Response {
	return SignedInvokeChaincode(stub, "userchannel", actions.Actions().UserCC().VerifyUserSignature, arg)
}

func SignedInvokeChaincode(stub ChaincodeStubInterfaceAdapter, channel string, preference actions.SecurityPreference, arg interface{}) pb.Response {
	signedRequest := requests.CCFullSignedRequest{
		UserSignature:   requests.UserSignature{},
		SystemSignature: requests.UserSignature{},
		Parameters: requests.CCParameters{
			UserParams: requests.CCParameter{},
			SystemParams: requests.CCParameter{
				UserID:   "NOT_APPLICABLE",
				ActionId: preference.GetActionID(),
				Param:    arg,
			},
		},
	}

	param, err := json.Marshal(signedRequest)
	if err != nil {
		return shim.Error(err.Error())
	}

	args := [][]byte{[]byte(preference.FunctionName), param}
	return stub.InvokeChaincode(preference.ChaincodeName, args, channel)
}

func SignedInvokeChaincodeWithUserId(stub ChaincodeStubInterfaceAdapter, userId string, channel string, preference actions.SecurityPreference, arg interface{}) pb.Response {
	signedRequest := requests.CCFullSignedRequest{
		UserSignature:   requests.UserSignature{},
		SystemSignature: requests.UserSignature{},
		Parameters: requests.CCParameters{
			UserParams: requests.CCParameter{},
			SystemParams: requests.CCParameter{
				UserID:   userId,
				ActionId: preference.GetActionID(),
				Param:    arg,
			},
		},
	}

	param, err := json.Marshal(signedRequest)
	if err != nil {
		return shim.Error(err.Error())
	}

	args := [][]byte{[]byte(preference.FunctionName), param}
	return stub.InvokeChaincode(preference.ChaincodeName, args, channel)
}